
/*******************************************************
*
* Copyright (c) 2003-2012 by University of Queensland
* Earth Systems Science Computational Center (ESSCC)
* http://www.uq.edu.au/esscc
*
* Primary Business: Queensland, Australia
* Licensed under the Open Software License version 3.0
* http://www.opensource.org/licenses/osl-3.0.php
*
*******************************************************/

/**************************************************************/

/*   Dudley: Mesh: NodeFile */

/* copies the array newX into self->coordinates */

/**************************************************************/

#include "NodeFile.h"
#include "Util.h"

/**************************************************************/

void Dudley_NodeFile_setCoordinates(Dudley_NodeFile * self, escriptDataC * newX)
{
    char error_msg[LenErrorMsg_MAX];
    size_t numDim_size;
    int n;
    if (getDataPointSize(newX) != self->numDim)
    {
	sprintf(error_msg, "Dudley_NodeFile_setCoordinates: dimension of new coordinates has to be %d.", self->numDim);
	Dudley_setError(VALUE_ERROR, error_msg);
    }
    else if (!numSamplesEqual(newX, 1, self->numNodes))
    {
	sprintf(error_msg, "Dudley_NodeFile_setCoordinates: number of given nodes must to be %d.", self->numNodes);
	Dudley_setError(VALUE_ERROR, error_msg);
    }
    else
    {
	numDim_size = self->numDim * sizeof(double);
	Dudley_increaseStatus(self);
#pragma omp parallel private(n)
	{

#pragma omp for schedule(static)
	    for (n = 0; n < self->numNodes; n++)
	    {
		memcpy(&(self->Coordinates[INDEX2(0, n, self->numDim)]), getSampleDataROFast(newX, n), numDim_size);
	    }
	}
    }
}
